// Copyright 2019 DeepMap, Inc.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
package codegen

import (
	"net/http"
	"testing"
)

func TestGenerateDefaultOperationID(t *testing.T) {
	type test struct {
		op string
		path string
		want string
		wantErr bool
	}

	suite := []test {
		{
			op: http.MethodGet,
			path: "/v1/foo/bar",
			want: "GetV1FooBar",
			wantErr: false,
		},
		{
			op: http.MethodGet,
			path: "/v1/foo/bar/",
			want: "GetV1FooBar",
			wantErr: false,
		},
		{
			op: http.MethodPost,
			path: "/v1",
			want: "PostV1",
			wantErr: false,
		},
		{
			op: http.MethodPost,
			path: "v1",
			want: "PostV1",
			wantErr: false,
		},
		{
			path: "v1",
			want: "",
			wantErr: true,
		},
		{
			path: "",
			want: "PostV1",
			wantErr: true,
		},
	}

	for _, test := range suite {
		got, err := generateDefaultOperationID(test.op, test.path)
		if err != nil {
			if !test.wantErr {
				t.Fatalf("did not expected error but got %v", err)
			}
		}

		if test.wantErr	{
			return
		}
		if got != test.want {
			t.Fatalf("Operation ID generation error. Want [%v] Got [%v]", test.want, got)
		}
	}
}